DEFAULT_PORT_HIGHSPEED: int = 10152
DEFAULT_PORT_FILTER: int = 14580

DEFAULT_HOST: str = "rotate.aprs.net"
DEFAULT_PORT: int = DEFAULT_PORT_HIGHSPEED
DEFAULT_CALLSIGN: str = "nocall"
DEFAULT_SERVER_FILTER: str = ""
DEFAULT_HEARTBEAT: int = 15
